import { 
  select, 
  csv, 
  scaleLinear,
  scaleTime,
  scaleOrdinal,
  max,
  extent,
  scaleBand,
  scalePoint,
  axisLeft,
  axisBottom,
  format,
  line,
  curveBasis,
  timeParse,
  nest,
  schemeCategory10
} from 'd3';

import { colorLegend } from './colorLegend';

const renderBar = data => {
  const svg = select('svg.bar');
	const height = +svg.attr('height');
	const width = +svg.attr('width');
  
  const xValue = d => d.population
  const yValue = d => d.Country
  const margin = {top: 15, right: 50, bottom: 20, left:120}
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
	const xScale = scaleLinear()
  	.domain([0, max(data, xValue)])
  	.range([0, innerWidth]);
  
  console.log(max(data, xValue));
  
  const yScale = scaleBand()
  	.domain(data.map( yValue))
  	.range([0, innerHeight])
  .padding(0.1);
  
  const g = svg.append('g')
  	.attr('transform',`translate(${margin.left}, ${margin.top})`); 
    
  const xAxisTickFormat = number =>
  format('.2s')(number)
  	.replace('G','B');
  
 	const xAxis = axisBottom(xScale)
 		.tickFormat(xAxisTickFormat)
  	.tickSize(-innerHeight)
  	; 
 
  g.append('g')
    .call(axisLeft(yScale))
  		.selectAll('.domain, .tick line')
  			.remove(); 
  
  const xAxisG = g.append('g')
    .call(xAxis)
  	.attr('transform',`translate(0,${innerHeight})`);
  xAxisG.select('.domain').remove();
  
  g.selectAll('rect').data(data)
  	.enter().append('rect')
  		.attr('y', d => yScale(yValue(d)))	
    	.attr('width', d => xScale(xValue(d)))
  		.attr('height', yScale.bandwidth());
  
  // g.append('text')
  // 	.text('Top 10 Most Populous countries')
  // 	.attr('y', -10)
 
  // xAxisG.append('text')
  // 	.attr('class','axisTitle')
  // 	.text('Population')
  // 	.attr('fill','black')
  //   .attr('y', 50)
  // 	.attr('x',innerWidth/2)

}; 
// Bar chart render Finish here



const renderScatter = data => {
  const titleText = 'Top 10 Most Populous Countries';
	const xAxisLabelText = 'Population';
	const svg = select('svg.scatter');
	console.log(svg)
	const width = +svg.attr('width');
	const height = +svg.attr('height');
  
  const xValue = d => d.population;
  const yValue = d => d.Country;
  const margin = { top: 15, right: 50, bottom: 20, left: 100 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth])
  	.nice();
  
  const yScale = scalePoint()
    .domain(data.map(yValue))
    .range([0, innerHeight])
    .padding(0.7);
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxisTickFormat = number =>
    format('.2s')(number)
      .replace('G', 'B');
  
  const xAxis = axisBottom(xScale)
    .tickFormat(xAxisTickFormat)
    .tickSize(-innerHeight);
  
  const yAxis = axisLeft(yScale)
  	.tickSize(-innerWidth);
  
  g.append('g')
    .call(yAxis)
    .selectAll('.domain')
      .remove();
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  // xAxisG.append('text')
  //     .attr('class', 'axis-label')
  //     .attr('y', 65)
  //     .attr('x', innerWidth / 2)
  //     .attr('fill', 'black')
  //     .text(xAxisLabelText);
  
  g.selectAll('circle').data(data)
    .enter().append('circle')
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', 4);
  
  // g.append('text')
  //     .attr('class', 'title')
  //     .attr('y', -10)
  //     .text(titleText);
};
// Render Scatter ends here



const renderLine = data => {
  const svg = select('svg.line');

	const width = +svg.attr('width');
	const height = +svg.attr('height');
  
  const title = 'Line Chart';
  
  const xValue = d => d.Year;
  const xAxisLabel = 'Year';
  
  const yValue = d => d.Population;
  const circleRadius = 3;
  const yAxisLabel = 'Population Size';
  
  const colorValue = d => d.Country;
  
  const margin = { top: 10, right: 40, bottom: 150, left: 105 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])

  
  const yScale = scaleLinear()
    .domain([0, max(data, yValue)])
    // .domain([45, 110])
    .range([innerHeight, 0])
    .nice();
  
  const colorScale = scaleOrdinal(schemeCategory10);
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
    const yAxisTickFormat = number =>
  			format('.2s')(number)
  			.replace('G','B');
  
  const yAxis = axisLeft(yScale)
  	.tickFormat(yAxisTickFormat)
    .tickSize(-innerWidth)
    .tickPadding(10);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -60)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 80)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  const lineGenerator = line()
    .x(d => xScale(xValue(d)))
    .y(d => yScale(yValue(d)))
    .curve(curveBasis);
  
  const nestedData = nest()
    .key(colorValue)
    .entries(data);
  colorScale.domain(nestedData.map(d => d.key))
  
  g.selectAll('.line-path').data(nestedData)
    .enter().append('path')
      .attr('class', 'line-path')
      .attr('d', d => lineGenerator(d.values))
      .attr('stroke', d => colorScale(d.key));
  
  // g.append('text')
  //     .attr('class', 'title')
  //     .attr('y', -10)
  //     .text(title);

  svg.append('g')
    .attr('transform', `translate(120,-4)`)
  	.attr('class', `legendText`)
    .call(colorLegend, {
      colorScale,
      circleRadius: 4,
      spacing: 12,
      textOffset: 14
    });

};
// Multi Line ends here

function LineInteractive(data) {
// set the dimensions and margins of the graph
var margin = {top: 10, right: 30, bottom: 30, left: 60},
    width = 880 - margin.left - margin.right,
    height = 250 - margin.top - margin.bottom;

// append the svg object to the body of the page
var svg = d3.select("#my_dataviz")
  .append("svg")
    .attr("width", width + margin.left + margin.right)
    .attr("height", height + margin.top + margin.bottom)
  .append("g")
    .attr("transform",
          "translate(" + margin.left + "," + margin.top + ")");

  	
  	var parser = d3.timeParse("%Y");
    // List of groups (here I have one group per column)
    var allGroup = d3.map(data, function(d){return(d.Country)}).keys()

    // add the options to the button
    d3.select("#selectButton")
      .selectAll('myOptions')
     	.data(allGroup)
      .enter()
    	.append('option')
      .text(function (d) { return d; }) // text showed in the menu
      .attr("value", function (d) { return d; }) // corresponding value returned by the button

    // A color scale: one color for each group
    var myColor = d3.scaleOrdinal()
      .domain(allGroup)
      .range(d3.schemeSet2);

    // Add X axis --> it is a date format
    var x = d3.scaleLinear()
      .domain(d3.extent(data, function(d) { return d.Year; }))
      .range([ 0, width ]);
    var xAxis = svg.append("g")
      .attr("transform", "translate(0," + height + ")")
      .call(d3.axisBottom(x).tickFormat(function(d) {return d }));
  
  	xAxis.append('text')
      .attr('class', 'axis-label')
      .attr('y', 30)
      .attr('x', 400)
      .attr('fill', 'black')
      .text("Year");

    // Add Y axis
    var y = d3.scaleLinear()
      .domain([0, d3.max(data, function(d) { return +d.Population; })])
      .range([ height, 0 ]);
    var yAxis = svg.append("g")
      .call(d3.axisLeft(y).tickFormat(function(n){ return d3.format('.2s')(n).replace('G','B')}));
  
  yAxis.append('text')
      .attr('class', 'axis-label')
      .attr('y', -40)
      .attr('x', -200 / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text("Population");
    // Initialize line with first group of the list
    var line = svg
      .append('g')
      .append("path")
        .datum(data.filter(function(d){return d.Country==allGroup[0]}))
        .attr("d", d3.line()
          .x(function(d) { return x(d.Year) })
          .y(function(d) { return y(+d.Population) })
        )
        .attr("stroke", function(d){ return myColor("valueA") })
        .style("stroke-width", 4)
        .style("fill", "none")

    // A function that update the chart
    function update(selectedGroup) {

      // Create new data with the selection?
      var dataFilter = data.filter(function(d){return d.Country==selectedGroup})

      // Give these new data to update line
      line
          .datum(dataFilter)
          .transition()
          .duration(1000)
          .attr("d", d3.line()
            .x(function(d) { return x(d.Year) })
            .y(function(d) { return y(+d.Population) })
          )
          .attr("stroke", function(d){ return myColor(selectedGroup) })
    }

    // When the button is changed, run the updateChart function
    d3.select("#selectButton").on("change", function(d) {
        // recover the option that has been chosen
        var selectedOption = d3.select(this).property("value")
        // run the updateChart function with this selected option
        update(selectedOption)
    })

}



csv('data.csv').then(data => {
	data.forEach(d => {
  	d.population = +d.population * 1000;
  });
  renderBar(data);
  renderScatter(data);
});

// csv('yearly-population.csv')
//   .then(data => {
//   const parseDate = timeParse("%d/%m/%Y");
//     data.forEach(d => {
//       d.Population = +d.Population;
//       d.Year = new Date(d.Year);
//     });
//     renderLine(data);
//   });

d3.csv("yearly-population.csv")
  .then(function(data) {
  	data.forEach(function(d) {
      d.Year = d.Year;
      d.Population = d.Population;
      d.Country = d.Country;
  });
  LineInteractive(data);

})
